@@ -1,15 +1,16 @@
 use std::cmp::Ordering;
 use std::collections::{HashMap, HashSet};
 use std::convert::TryInto;
-use std::process;
+use std::io::{BufWriter, Write};
 use std::sync::atomic::{AtomicBool, Ordering as AOrdering};
+use std::{io, process};
 
 use globset::GlobBuilder;
 use log::error;
 
 use crate::cmd::opts::{ListCmd, *};
 use crate::cmd::table::{Alignment, AsRow, Column, ColumnId, Row, Separator, TableView, TextCell};
-use crate::cmd::{paint, println, style, Cmd, GlobalArgs, DATE_FORMAT};
+use crate::cmd::{paint, style, Cmd, GlobalArgs, DATE_FORMAT};
 use crate::config::Config;
 use crate::ocfl::{
     FileDetails, InventoryPath, LogicalPath, ObjectVersion, ObjectVersionDetails, OcflRepo, Result,
@@ -57,10 +58,11 @@ impl ListCmd {
             // It's safe to stream the results so long as they are not sorted and do not need
             // to be displayed in a table
             self.stream_objects(args, iter);
-            Ok(())
         } else {
-            self.write_objects_to_table(args, terminate, iter)
+            self.write_objects_to_table(args, terminate, iter);
         }
+
+        Ok(())
     }
 
     fn stream_objects<'a>(
@@ -68,12 +70,13 @@ impl ListCmd {
         args: GlobalArgs,
         iter: Box<dyn Iterator<Item = Result<ObjectVersionDetails>> + 'a>,
     ) {
+        let mut out = BufWriter::new(io::stdout());
+        let isatty = atty::is(atty::Stream::Stdout);
         let mut has_errors = false;
         let mut header_printed = false;
 
         for object in iter {
             if let Err(e) = object {
-                println!("here");
                 has_errors = true;
                 error!("{:#}", e);
                 continue;
@@ -101,7 +104,7 @@ impl ListCmd {
                         .push_str(&paint(args.no_styles, *style::UNDERLINE, VERSION).to_string());
                 }
 
-                println(header_line);
+                let _ = writeln!(out, "{}", header_line);
             }
 
             let object = object.unwrap();
@@ -139,9 +142,14 @@ impl ListCmd {
                 line.push_str(&object.object_root);
             }
 
-            println(line);
+            let _ = writeln!(out, "{}", line);
+            if isatty {
+                let _ = out.flush();
+            }
         }
 
+        let _ = out.flush();
+
         if has_errors {
             process::exit(1);
         }
@@ -152,7 +160,7 @@ impl ListCmd {
         args: GlobalArgs,
         terminate: &AtomicBool,
         iter: Box<dyn Iterator<Item = Result<ObjectVersionDetails>> + 'a>,
-    ) -> Result<()> {
+    ) {
         let mut has_errors = false;
         let mut objects = Vec::new();
 
@@ -164,7 +172,7 @@ impl ListCmd {
             }
 
             if terminate.load(AOrdering::Acquire) {
-                return Ok(());
+                return;
             }
             objects.push(object.unwrap());
         }
@@ -178,25 +186,26 @@ impl ListCmd {
         });
 
         if terminate.load(AOrdering::Acquire) {
-            return Ok(());
+            return;
         }
 
         let mut table = self.object_table(args);
 
         for object in &objects {
             if terminate.load(AOrdering::Acquire) {
-                return Ok(());
+                return;
             }
 
             table.add_row(object);
         }
 
-        table.write_stdio()?;
+        let out = io::stdout();
+        let mut writer = BufWriter::new(out.lock());
+        let _ = table.write(&mut writer);
+        let _ = writer.flush();
 
         if has_errors {
             process::exit(1);
-        } else {
-            Ok(())
         }
     }
 
@@ -225,7 +234,12 @@ impl ListCmd {
 
         let mut table = self.object_content_table(args);
         listings.iter().for_each(|listing| table.add_row(listing));
-        Ok(table.write_stdio()?)
+
+        let out = io::stdout();
+        let mut writer = BufWriter::new(out.lock());
+        let _ = table.write(&mut writer);
+
+        Ok(())
     }
 
     fn object_table(&self, args: GlobalArgs) -> TableView {
