@@ -1,42 +1,44 @@
 use std::borrow::Cow;
+use std::collections::hash_map::Iter;
 use std::collections::{BTreeMap, HashMap, HashSet};
-use std::convert::TryInto;
+use std::convert::{TryFrom, TryInto};
 use std::mem;
 use std::rc::Rc;
+use std::str::FromStr;
 
 use chrono::{DateTime, Local};
 use globset::GlobBuilder;
-use log::error;
 use once_cell::unsync::OnceCell;
 use serde::{Deserialize, Serialize};
 
-use crate::ocfl::bimap::{IntoIter, Iter, PathBiMap};
-use crate::ocfl::consts::{DEFAULT_CONTENT_DIR, INVENTORY_TYPE, MUTABLE_HEAD_EXT_DIR};
+use crate::ocfl::bimap::PathBiMap;
+use crate::ocfl::consts::{DEFAULT_CONTENT_DIR, INVENTORY_TYPE};
 use crate::ocfl::digest::{DigestAlgorithm, HexDigest};
 use crate::ocfl::error::{not_found, not_found_path, Result, RocflError};
-use crate::ocfl::{CommitMeta, Diff, InventoryPath, VersionNum};
+use crate::ocfl::{
+    validate, CommitMeta, ContentPath, ContentPathVersion, Diff, InventoryPath, LogicalPath,
+    VersionNum, VersionRef,
+};
 
 const STAGING_MESSAGE: &str = "Staging new version";
 const ROCFL_USER: &str = "rocfl";
 const ROCFL_ADDRESS: &str = "https://github.com/pwinckles/rocfl";
 
-// TODO need to lock down all of these public members
-
 /// OCFL inventory serialization object
-#[derive(Deserialize, Serialize, Debug)]
+#[derive(Serialize, Debug)]
 #[serde(rename_all = "camelCase")]
 pub struct Inventory {
     pub id: String,
     #[serde(rename = "type")]
     pub type_declaration: String,
+    // TODO this would be better as a specific type that only allows sha256/sha512 -- but is a bit of a pain to change
     pub digest_algorithm: DigestAlgorithm,
     pub head: VersionNum,
     pub content_directory: Option<String>,
-    // TODO look into deduping all HexDigests and InventoryPaths using a deserialize seed
-    manifest: PathBiMap,
+    manifest: PathBiMap<ContentPath>,
     pub versions: BTreeMap<VersionNum, Version>,
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub fixity: Option<HashMap<String, HashMap<String, Vec<String>>>>,
+    fixity: Option<HashMap<String, HashMap<String, Vec<String>>>>,
 
     #[serde(skip)]
     /// Path to the object's root relative the storage root. This path should use `/` as
@@ -59,17 +61,15 @@ pub struct InventoryBuilder {
     digest_algorithm: DigestAlgorithm,
     head: VersionNum,
     content_directory: String,
-    manifest: PathBiMap,
+    manifest: PathBiMap<ContentPath>,
     versions: BTreeMap<VersionNum, Version>,
-    object_root: String,
-    storage_path: String,
 }
 
 /// OCFL version serialization object
-#[derive(Deserialize, Serialize, Debug, Clone)]
+#[derive(Serialize, Debug, Clone)]
 pub struct Version {
     pub created: DateTime<Local>,
-    state: PathBiMap,
+    state: PathBiMap<LogicalPath>,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub message: Option<String>,
     #[serde(skip_serializing_if = "Option::is_none")]
@@ -77,17 +77,57 @@ pub struct Version {
 
     /// All of the logical path parts that should be treated as directories
     #[serde(skip)]
-    logical_dirs: OnceCell<HashSet<InventoryPath>>,
+    logical_dirs: OnceCell<HashSet<LogicalPath>>,
 }
 
 /// OCFL user serialization object
-#[derive(Deserialize, Serialize, Debug, Clone)]
+#[derive(Deserialize, Serialize, Debug, Clone, Eq, PartialEq)]
 pub struct User {
     pub name: Option<String>,
     pub address: Option<String>,
 }
 
 impl Inventory {
+    /// Creates a new inventory, this is intended for deserialization
+    #[allow(clippy::too_many_arguments)]
+    pub fn new(
+        id: String,
+        type_declaration: String,
+        digest_algorithm: DigestAlgorithm,
+        head: VersionNum,
+        content_directory: Option<String>,
+        manifest: PathBiMap<ContentPath>,
+        versions: BTreeMap<VersionNum, Version>,
+        fixity: Option<HashMap<String, HashMap<String, Vec<String>>>>,
+    ) -> Result<Self> {
+        validate::validate_object_id(&id)?;
+        validate::validate_digest_algorithm(digest_algorithm)?;
+        if let Some(dir) = &content_directory {
+            validate::validate_content_dir(dir)?
+        }
+
+        if !versions.contains_key(&head) {
+            return Err(RocflError::CorruptObject {
+                object_id: id,
+                message: format!("HEAD version {} was not found", head),
+            });
+        }
+
+        Ok(Self {
+            id,
+            type_declaration,
+            digest_algorithm,
+            head,
+            content_directory,
+            manifest,
+            versions,
+            fixity,
+            object_root: Default::default(),
+            storage_path: Default::default(),
+            mutable_head: false,
+        })
+    }
+
     /// Returns a new inventory builder
     pub fn builder(object_id: &str) -> InventoryBuilder {
         InventoryBuilder::new(object_id)
@@ -137,10 +177,15 @@ impl Inventory {
     }
 
     /// Returns true if the path exists in the manifest
-    pub fn contains_content_path(&self, content_path: &InventoryPath) -> bool {
+    pub fn contains_content_path(&self, content_path: &ContentPath) -> bool {
         self.manifest.contains_path(content_path)
     }
 
+    /// Returns the digest associated to the content path in the manifest
+    pub fn digest_for_content_path(&self, content_path: &ContentPath) -> Option<&Rc<HexDigest>> {
+        self.manifest.get_id(content_path)
+    }
+
     /// Returns the first content path associated with the specified digest, or an error if it does
     /// not exist.
     ///
@@ -148,36 +193,27 @@ impl Inventory {
     /// or earlier.
     ///
     /// If `logical_path` is specified and multiple content paths for the digest are found, then
-    /// then the path that maps directly to the logical path is selected or the first if none match.
+    /// the path that maps directly to the logical path is selected or the first if none match.
     pub fn content_path_for_digest(
         &self,
         digest: &HexDigest,
-        version_num: Option<VersionNum>,
-        logical_path: Option<&InventoryPath>,
-    ) -> Result<&Rc<InventoryPath>> {
-        let version_num = version_num.unwrap_or(self.head);
+        version_num: VersionRef,
+        logical_path: Option<&LogicalPath>,
+    ) -> Result<&Rc<ContentPath>> {
+        let version_num = version_num.resolve(self.head);
 
         match self.manifest.get_paths(digest) {
             Some(paths) => {
                 let mut matches = Vec::new();
 
                 for path in paths {
-                    // TODO move this to `ContentPath` after it's created
-                    if let Some(slash) = path.as_ref().as_ref().find('/') {
-                        let version_str = &path.as_ref().as_ref().as_str()[0..slash];
+                    let current_version = match path.version {
+                        ContentPathVersion::VersionNum(current_version) => current_version,
+                        ContentPathVersion::MutableHead => self.head,
+                    };
 
-                        // Unfortunately, mutable head extension paths do not contain a version num
-                        let version = if self.mutable_head
-                            && path.as_ref().as_ref().starts_with(MUTABLE_HEAD_EXT_DIR)
-                        {
-                            self.head
-                        } else {
-                            version_str.try_into()?
-                        };
-
-                        if version <= version_num {
-                            matches.push(path);
-                        }
+                    if current_version <= version_num {
+                        matches.push(path);
                     }
                 }
 
@@ -192,8 +228,8 @@ impl Inventory {
                         self.defaulted_content_dir(),
                         logical_path.unwrap()
                     );
-                    for path in matches.iter() {
-                        if path.as_ref().as_ref().ends_with(&suffix) {
+                    for path in &matches {
+                        if path.ends_with(&suffix) {
                             return Ok(path);
                         }
                     }
@@ -212,13 +248,13 @@ impl Inventory {
     /// is not found.
     pub fn content_path_for_logical_path(
         &self,
-        logical_path: &InventoryPath,
-        version_num: Option<VersionNum>,
-    ) -> Result<&Rc<InventoryPath>> {
-        let version_num = version_num.unwrap_or(self.head);
+        logical_path: &LogicalPath,
+        version_num: VersionRef,
+    ) -> Result<&Rc<ContentPath>> {
+        let version_num = version_num.resolve(self.head);
         let version = self.get_version(version_num)?;
 
-        let digest = match version.lookup_digest(&logical_path) {
+        let digest = match version.lookup_digest(logical_path) {
             Some(digest) => digest,
             None => {
                 return Err(RocflError::NotFound(format!(
@@ -228,7 +264,24 @@ impl Inventory {
             }
         };
 
-        self.content_path_for_digest(digest, Some(version_num), Some(logical_path))
+        self.content_path_for_digest(digest, version_num.into(), Some(logical_path))
+    }
+
+    /// Returns a reference to the set of all of the content paths that are associated to the
+    /// digest
+    pub fn content_paths(&self, digest: &HexDigest) -> Option<&HashSet<Rc<ContentPath>>> {
+        self.manifest.get_paths(digest)
+    }
+
+    /// Returns a set of all of the content paths in the inventory
+    pub fn all_content_paths(&self) -> HashSet<Rc<ContentPath>> {
+        let mut paths = HashSet::with_capacity(self.manifest.len());
+
+        for (path, _) in &self.manifest {
+            paths.insert(path.clone());
+        }
+
+        paths
     }
 
     /// Returns the diffs of two versions. An error is returned if either of the specified versions
@@ -257,16 +310,16 @@ impl Inventory {
 
     /// Dedups all of the content paths that were added in the most recent version. All of the
     /// paths that are removed from the manifest are returned.
-    pub fn dedup_head(&mut self) -> Vec<Rc<InventoryPath>> {
+    pub fn dedup_head(&mut self) -> Vec<Rc<ContentPath>> {
         let mut removed = Vec::new();
         let prefix = format!("{}/", self.head.to_string());
 
-        let mut matches: HashMap<Rc<HexDigest>, HashSet<Rc<InventoryPath>>> = HashMap::new();
+        let mut matches: HashMap<Rc<HexDigest>, HashSet<Rc<ContentPath>>> = HashMap::new();
 
         for (digest, paths) in self.manifest.iter_id_paths() {
             if paths.len() > 1 {
                 for path in paths {
-                    if path.as_ref().as_ref().starts_with(&prefix) {
+                    if path.starts_with(&prefix) {
                         matches
                             .entry(digest.clone())
                             .or_insert_with(HashSet::new)
@@ -306,17 +359,13 @@ impl Inventory {
     /// Content paths are NOT deduped until the version is committed.
     ///
     /// If the logical path already exists in the version, then the existing file is overwritten.
-    pub fn add_file_to_head(
-        &mut self,
-        digest: HexDigest,
-        logical_path: InventoryPath,
-    ) -> Result<()> {
+    pub fn add_file_to_head(&mut self, digest: HexDigest, logical_path: LogicalPath) -> Result<()> {
         let digest_rc = match self.manifest.get_id_rc(&digest) {
             Some(digest_rc) => digest_rc.clone(),
             None => Rc::new(digest),
         };
 
-        let content_path = self.new_content_path_head(&logical_path)?;
+        let content_path = self.new_content_path(&logical_path);
         self.manifest
             .insert_rc(digest_rc.clone(), Rc::new(content_path));
 
@@ -328,8 +377,8 @@ impl Inventory {
     pub fn copy_file_to_head(
         &mut self,
         src_version_num: VersionNum,
-        src_path: &InventoryPath,
-        dst_path: InventoryPath,
+        src_path: &LogicalPath,
+        dst_path: LogicalPath,
     ) -> Result<()> {
         let src_version = self.get_version(src_version_num)?;
         let digest = match src_version.lookup_digest(src_path) {
@@ -344,8 +393,8 @@ impl Inventory {
     /// path is validated prior to the move.
     pub fn move_file_in_head(
         &mut self,
-        src_path: &InventoryPath,
-        dst_path: InventoryPath,
+        src_path: &LogicalPath,
+        dst_path: LogicalPath,
     ) -> Result<()> {
         let head = self.head_version_mut();
         let digest = match head.lookup_digest(src_path) {
@@ -368,18 +417,18 @@ impl Inventory {
     pub fn move_new_in_head_file(
         &mut self,
         digest: HexDigest,
-        src_path: &InventoryPath,
-        dst_path: InventoryPath,
+        src_path: &LogicalPath,
+        dst_path: LogicalPath,
     ) -> Result<()> {
         let digest_rc = match self.manifest.get_id_rc(&digest) {
             Some(digest_rc) => digest_rc.clone(),
             None => Rc::new(digest),
         };
 
-        let src_content_path = self.new_content_path_head(&src_path)?;
+        let src_content_path = self.new_content_path(src_path);
         self.manifest.remove_path(&src_content_path);
 
-        let content_path = self.new_content_path_head(&dst_path)?;
+        let content_path = self.new_content_path(&dst_path);
         self.manifest
             .insert_rc(digest_rc.clone(), Rc::new(content_path));
 
@@ -395,19 +444,15 @@ impl Inventory {
     /// and returned.
     pub fn remove_logical_path_from_head(
         &mut self,
-        logical_path: &InventoryPath,
-    ) -> Option<InventoryPath> {
+        logical_path: &LogicalPath,
+    ) -> Option<ContentPath> {
         let head = self.head_version_mut();
 
         if head.remove_file(logical_path).is_some() {
             // Remove the path from the manifest if it was added in the HEAD version
-            match self.new_content_path_head(logical_path) {
-                Ok(content_path) => {
-                    if self.manifest.remove_path(&content_path).is_some() {
-                        return Some(content_path);
-                    }
-                }
-                Err(e) => error!("Failed to create content path for {}: {}", logical_path, e),
+            let content_path = self.new_content_path(logical_path);
+            if self.manifest.remove_path(&content_path).is_some() {
+                return Some(content_path);
             }
         }
 
@@ -416,26 +461,12 @@ impl Inventory {
 
     /// Returns a new content path for the specified logical path, assuming a direct one-to-one
     /// mapping of logical path to content path.
-    pub fn new_content_path_head(&self, logical_path: &InventoryPath) -> Result<InventoryPath> {
-        self.new_content_path(self.head, logical_path)
-    }
-
-    /// Returns a new content path for the specified logical path, assuming a direct one-to-one
-    /// mapping of logical path to content path.
-    pub fn new_content_path(
-        &self,
-        version_num: VersionNum,
-        logical_path: &InventoryPath,
-    ) -> Result<InventoryPath> {
-        format!(
-            "{}/{}/{}",
-            version_num.to_string(),
-            self.defaulted_content_dir(),
-            logical_path.as_ref()
-        )
-        .try_into()
+    pub fn new_content_path(&self, logical_path: &LogicalPath) -> ContentPath {
+        logical_path.to_content_path(self.head, self.defaulted_content_dir())
     }
 
+    /// Returns the content directory specified in the inventory or the default value if none
+    /// is specified.
     pub fn defaulted_content_dir(&self) -> &str {
         match &self.content_directory {
             Some(dir) => dir.as_str(),
@@ -443,16 +474,69 @@ impl Inventory {
         }
     }
 
-    /// Performs a spot check on the inventory to see if it appears valid. This is not an
-    /// exhaustive check, and does not guarantee that the inventory is valid.
-    pub fn validate(&self) -> Result<()> {
-        if !self.versions.contains_key(&self.head) {
-            return Err(RocflError::CorruptObject {
-                object_id: self.id.clone(),
-                message: format!("HEAD version {} was not found", self.head),
-            });
+    /// Returns a reference to the inventory's manifest
+    pub fn manifest(&self) -> &PathBiMap<ContentPath> {
+        &self.manifest
+    }
+
+    /// Inverts the fixity block and returns a map of content paths to their expected digests
+    #[allow(clippy::type_complexity)]
+    pub fn invert_fixity(
+        &self,
+    ) -> Option<HashMap<ContentPath, Vec<(DigestAlgorithm, Rc<HexDigest>)>>> {
+        if let Some(fixity) = &self.fixity {
+            let mut inverted = HashMap::new();
+
+            for (algorithm, manifest) in fixity {
+                // TODO skipping blake2b until we can support streaming them
+                if algorithm.starts_with("blake2b") {
+                    continue;
+                }
+                if let Ok(algorithm) = DigestAlgorithm::from_str(algorithm) {
+                    for (digest, paths) in manifest {
+                        let digest = Rc::new(HexDigest::from(digest.as_str()));
+                        for path in paths {
+                            inverted
+                                .entry(ContentPath::try_from(path).unwrap())
+                                .or_insert_with(Vec::new)
+                                .push((algorithm, digest.clone()))
+                        }
+                    }
+                }
+            }
+
+            Some(inverted)
+        } else {
+            None
         }
-        Ok(())
+    }
+
+    /// Returns a set of all of the content paths in the manifest
+    pub fn manifest_paths(&self) -> HashSet<Rc<ContentPath>> {
+        let mut paths = HashSet::with_capacity(self.manifest.len());
+
+        for (content_path, _) in &self.manifest {
+            paths.insert(content_path.clone());
+        }
+
+        paths
+    }
+
+    /// Returns a set of all of the content paths in the fixity block
+    pub fn fixity_paths(&self) -> HashSet<&str> {
+        let mut paths = HashSet::new();
+
+        if let Some(fixity) = &self.fixity {
+            fixity
+                .values()
+                .flat_map(|m| m.values())
+                .flat_map(|v| v.iter())
+                .for_each(|path| {
+                    paths.insert(path.as_ref());
+                });
+        }
+
+        paths
     }
 }
 
@@ -462,12 +546,10 @@ impl InventoryBuilder {
             id: object_id.to_string(),
             type_declaration: INVENTORY_TYPE.to_string(),
             digest_algorithm: DigestAlgorithm::Sha512,
-            head: VersionNum::with_width(1, 0),
+            head: VersionNum::v1(),
             content_directory: DEFAULT_CONTENT_DIR.to_string(),
             manifest: PathBiMap::new(),
             versions: BTreeMap::new(),
-            object_root: "".to_string(),
-            storage_path: "".to_string(),
         }
     }
 
@@ -489,27 +571,36 @@ impl InventoryBuilder {
     pub fn build(mut self) -> Result<Inventory> {
         self.versions.insert(self.head, Version::new_staged());
 
-        let inventory = Inventory {
-            id: self.id,
-            type_declaration: self.type_declaration,
-            digest_algorithm: self.digest_algorithm,
-            head: self.head,
-            content_directory: Some(self.content_directory),
-            manifest: self.manifest,
-            versions: self.versions,
-            fixity: None,
-            object_root: self.object_root,
-            storage_path: self.storage_path,
-            mutable_head: false,
-        };
-
-        inventory.validate()?;
-
-        Ok(inventory)
+        Inventory::new(
+            self.id,
+            self.type_declaration,
+            self.digest_algorithm,
+            self.head,
+            Some(self.content_directory),
+            self.manifest,
+            self.versions,
+            None,
+        )
     }
 }
 
 impl Version {
+    /// Creates a new version, intended to be used for deserialization
+    pub fn new(
+        created: DateTime<Local>,
+        state: PathBiMap<LogicalPath>,
+        message: Option<String>,
+        user: Option<User>,
+    ) -> Self {
+        Self {
+            created,
+            state,
+            message,
+            user,
+            logical_dirs: Default::default(),
+        }
+    }
+
     /// Create a new Version initialized with values for staging
     pub fn new_staged() -> Self {
         Self::staged_version(PathBiMap::new())
@@ -520,7 +611,7 @@ impl Version {
         Self::staged_version(self.state.clone())
     }
 
-    fn staged_version(state: PathBiMap) -> Self {
+    fn staged_version(state: PathBiMap<LogicalPath>) -> Self {
         Self {
             created: Local::now(),
             message: Some(STAGING_MESSAGE.to_string()),
@@ -542,18 +633,23 @@ impl Version {
         self.created = meta.created.unwrap_or_else(Local::now);
     }
 
-    /// Returns a consuming iterator for the version's state
-    pub fn state_into_iter(&mut self) -> IntoIter {
-        self.remove_state().into_iter()
-    }
-
     /// Returns non-consuming iterator for the version's state
-    pub fn state_iter(&self) -> Iter {
+    pub fn state_iter(&self) -> Iter<Rc<LogicalPath>, Rc<HexDigest>> {
         self.state.iter()
     }
 
+    pub fn logical_paths(&self) -> HashSet<Rc<LogicalPath>> {
+        let mut paths = HashSet::with_capacity(self.state.len());
+
+        for (path, _) in &self.state {
+            paths.insert(path.clone());
+        }
+
+        paths
+    }
+
     /// Moves the current state map out, replacing it when an empty state
-    pub fn remove_state(&mut self) -> PathBiMap {
+    pub fn remove_state(&mut self) -> PathBiMap<LogicalPath> {
         if self.logical_dirs.get().is_some() {
             self.logical_dirs = OnceCell::default();
         }
@@ -562,22 +658,22 @@ impl Version {
 
     /// Returns a reference to the digest associated to a logical path, or None if the logical
     /// path does not exist in the version's state.
-    pub fn lookup_digest(&self, logical_path: &InventoryPath) -> Option<&Rc<HexDigest>> {
+    pub fn lookup_digest(&self, logical_path: &LogicalPath) -> Option<&Rc<HexDigest>> {
         self.state.get_id(logical_path)
     }
 
     /// Returns true if the specified path exists as either a logical file or directory
-    pub fn exists(&self, path: &InventoryPath) -> bool {
-        self.is_file(&path) || self.is_dir(path)
+    pub fn exists(&self, path: &LogicalPath) -> bool {
+        self.is_file(path) || self.is_dir(path)
     }
 
     /// Returns true if the specified path exists and is a logical file
-    pub fn is_file(&self, path: &InventoryPath) -> bool {
+    pub fn is_file(&self, path: &LogicalPath) -> bool {
         self.state.contains_path(path)
     }
 
     // Returns true if the specified path exists and is a logical directory
-    pub fn is_dir(&self, path: &InventoryPath) -> bool {
+    pub fn is_dir(&self, path: &LogicalPath) -> bool {
         self.get_logical_dirs().contains(path)
     }
 
@@ -589,8 +685,8 @@ impl Version {
     /// Returns an error if the specified path conflicts with the existing state.
     /// A path conflicts if it a portion of the path is interpreted as both a directory
     /// and a file.
-    pub fn validate_non_conflicting(&self, path: &InventoryPath) -> Result<()> {
-        if self.is_dir(&path) {
+    pub fn validate_non_conflicting(&self, path: &LogicalPath) -> Result<()> {
+        if self.is_dir(path) {
             return Err(RocflError::IllegalState(format!(
                 "Conflicting logical path {}: This path is already in use as a directory",
                 path
@@ -610,7 +706,7 @@ impl Version {
     }
 
     /// Returns a set of all of the logical paths that match the provided glob pattern
-    pub fn resolve_glob(&self, glob: &str, recursive: bool) -> Result<HashSet<Rc<InventoryPath>>> {
+    pub fn resolve_glob(&self, glob: &str, recursive: bool) -> Result<HashSet<Rc<LogicalPath>>> {
         let mut matches = HashSet::new();
 
         // Logical paths do not have leading slashes
@@ -629,8 +725,8 @@ impl Version {
             .build()?
             .compile_matcher();
 
-        for (path, _digest) in self.state.iter() {
-            if matcher.is_match(path.as_ref().as_ref()) {
+        for (path, _digest) in &self.state {
+            if matcher.is_match(path.as_path()) {
                 matches.insert(path.clone());
             }
         }
@@ -638,9 +734,9 @@ impl Version {
         if recursive {
             for dir in self.get_logical_dirs() {
                 if (glob_trailing_slash && matcher.is_match(format!("{}/", dir)))
-                    || (!glob_trailing_slash && matcher.is_match(dir.as_ref()))
+                    || (!glob_trailing_slash && matcher.is_match(dir.as_path()))
                 {
-                    matches.extend(self.paths_with_prefix(dir.as_ref()));
+                    matches.extend(self.paths_with_prefix(dir.as_str()));
                 }
             }
         }
@@ -649,7 +745,7 @@ impl Version {
     }
 
     /// Returns a set of all of the logical dirs that match the glob
-    pub fn resolve_glob_to_dirs(&self, glob: &str) -> Result<HashSet<&InventoryPath>> {
+    pub fn resolve_glob_to_dirs(&self, glob: &str) -> Result<HashSet<&LogicalPath>> {
         let mut matches = HashSet::new();
 
         // Logical paths do not have leading slashes
@@ -662,7 +758,7 @@ impl Version {
             .compile_matcher();
 
         for dir in self.get_logical_dirs() {
-            if matcher.is_match(dir.as_ref()) {
+            if matcher.is_match(dir.as_str()) {
                 matches.insert(dir);
             }
         }
@@ -670,8 +766,8 @@ impl Version {
         Ok(matches)
     }
 
-    /// Returns a list of all of the paths that beging with the specified prefix
-    pub fn paths_with_prefix(&self, prefix: &str) -> Vec<Rc<InventoryPath>> {
+    /// Returns a list of all of the paths that begin with the specified prefix
+    pub fn paths_with_prefix(&self, prefix: &str) -> Vec<Rc<LogicalPath>> {
         let mut matches = Vec::new();
 
         let prefix = if !prefix.ends_with('/') && !prefix.is_empty() {
@@ -680,8 +776,8 @@ impl Version {
             prefix.into()
         };
 
-        for (path, _digest) in self.state.iter() {
-            if path.as_ref().as_ref().starts_with(prefix.as_ref()) {
+        for (path, _digest) in &self.state {
+            if path.starts_with(prefix.as_ref()) {
                 matches.push(path.clone());
             }
         }
@@ -694,13 +790,13 @@ impl Version {
     /// this version's paths are returned as Adds.
     pub fn diff(&self, other: Option<&Version>) -> Vec<Diff> {
         let mut diffs = Vec::new();
-        let mut deletes: HashMap<Rc<HexDigest>, Vec<Rc<InventoryPath>>> = HashMap::new();
+        let mut deletes: HashMap<Rc<HexDigest>, Vec<Rc<LogicalPath>>> = HashMap::new();
 
         if let Some(left) = other {
             let mut seen = HashSet::with_capacity(left.state.len());
 
             for (path, left_digest) in left.state_iter() {
-                match self.lookup_digest(&path) {
+                match self.lookup_digest(path) {
                     None => {
                         deletes
                             .entry(left_digest.clone())
@@ -761,7 +857,7 @@ impl Version {
 
     /// Adds a new logical path to the version, and updates the logical directory set, if needed.
     /// This path MUST be added to the inventory manifest separately for the inventory to be valid.
-    fn add_file(&mut self, digest: Rc<HexDigest>, logical_path: InventoryPath) -> Result<()> {
+    fn add_file(&mut self, digest: Rc<HexDigest>, logical_path: LogicalPath) -> Result<()> {
         self.validate_non_conflicting(&logical_path)?;
         if let Some(dirs) = self.logical_dirs.get_mut() {
             dirs.extend(create_logical_dirs(&logical_path));
@@ -772,7 +868,7 @@ impl Version {
     }
 
     /// Removes a logical path from the version's state
-    fn remove_file(&mut self, path: &InventoryPath) -> Option<(Rc<InventoryPath>, Rc<HexDigest>)> {
+    fn remove_file(&mut self, path: &LogicalPath) -> Option<(Rc<LogicalPath>, Rc<HexDigest>)> {
         // must invalidate the logical dirs
         if self.logical_dirs.get().is_some() {
             self.logical_dirs = OnceCell::default();
@@ -781,9 +877,9 @@ impl Version {
     }
 
     /// Initializes a HashSet containing all of the logical directories within a version.
-    fn get_logical_dirs(&self) -> &HashSet<InventoryPath> {
+    fn get_logical_dirs(&self) -> &HashSet<LogicalPath> {
         self.logical_dirs.get_or_init(|| {
-            let mut dirs: HashSet<InventoryPath> = HashSet::with_capacity(self.state.len());
+            let mut dirs: HashSet<LogicalPath> = HashSet::with_capacity(self.state.len());
             // Add the root path
             dirs.insert("/".try_into().unwrap());
 
@@ -805,11 +901,11 @@ impl User {
     }
 }
 
-fn create_logical_dirs(path: &InventoryPath) -> HashSet<InventoryPath> {
+fn create_logical_dirs(path: &LogicalPath) -> HashSet<LogicalPath> {
     let mut dirs = HashSet::new();
 
     let mut parent = path.parent();
-    while parent.as_ref() != "" {
+    while !parent.is_empty() {
         let next = parent.parent();
         dirs.insert(parent);
         parent = next;
