@@ -317,6 +317,7 @@ struct Dir<'a> {
 
 /// The result of deserializing an inventory
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 enum ParseResult {
     /// Indicates the inventory was successfully deserialized and no errors were detected. The
     /// `ParseValidationResult` may still contain warnings.
@@ -1201,7 +1202,7 @@ impl<S: Storage> Validator<S> {
 
         if files.contains(&Listing::file(INVENTORY_FILE)) {
             let inventory_path = paths::join(version_dir, INVENTORY_FILE);
-            let mut digester = inventory.digest_algorithm.writer(Vec::new()).unwrap();
+            let mut digester = inventory.digest_algorithm.writer(Vec::new());
             self.storage.read(&inventory_path, &mut digester)?;
 
             let digest = digester.finalize_hex();
