@@ -14,12 +14,13 @@ use common::*;
 use fs_extra::dir::CopyOptions;
 use rand::Rng;
 use rocfl::ocfl::{
-    CommitMeta, DigestAlgorithm, FileDetails, LayoutExtensionName, OcflRepo, RocflError,
-    StorageLayout, VersionNum,
+    CommitMeta, DigestAlgorithm, ErrorCode, FileDetails, LayoutExtensionName, OcflRepo,
+    ProblemLocation, RocflError, StorageLayout, ValidationError, VersionNum, VersionRef, WarnCode,
 };
 use rusoto_core::Region;
 use rusoto_s3::{
-    DeleteObjectRequest, GetObjectRequest, HeadObjectRequest, ListObjectsV2Request, S3Client, S3,
+    DeleteObjectRequest, GetObjectRequest, HeadObjectRequest, ListObjectsV2Request,
+    PutObjectRequest, S3Client, S3,
 };
 use tokio::io::AsyncReadExt;
 
@@ -114,13 +115,13 @@ fn create_new_object() {
             repo.commit(object_id, CommitMeta::new(), None, false)
                 .unwrap();
 
-            let object = repo.get_object(object_id, None).unwrap();
+            let object = repo.get_object(object_id, VersionRef::Head).unwrap();
 
             assert_eq!(1, object.state.len());
 
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("test.txt")).unwrap(),
+                object.state.get(&lpath("test.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/test.txt",
                 "cf80cd8aed482d5d1527d7dc72fceff84e6326592848447d2dc0b0e87dfc9a90",
@@ -193,7 +194,7 @@ fn create_and_update_object() {
 
             repo.copy_files_internal(
                 object_id,
-                Some(VersionNum::new(1)),
+                VersionNum::v1().into(),
                 &vec!["a/b/file3.txt"],
                 "/",
                 false,
@@ -201,7 +202,7 @@ fn create_and_update_object() {
             .unwrap();
             repo.copy_files_internal(
                 object_id,
-                Some(VersionNum::new(1)),
+                VersionNum::v1().into(),
                 &vec!["a/file1.txt"],
                 "something/file1.txt",
                 false,
@@ -235,55 +236,55 @@ fn create_and_update_object() {
             repo.commit(object_id, CommitMeta::new(), None, false)
                 .unwrap();
 
-            let object = repo.get_object(object_id, None).unwrap();
+            let object = repo.get_object(object_id, VersionRef::Head).unwrap();
 
             assert_eq!(7, object.state.len());
 
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("file3.txt")).unwrap(),
+                object.state.get(&lpath("file3.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/a/b/file3.txt",
                 "e18fad97c1b6512b1588a1fa2b7f9a0e549df9cfc538ce6943b4f0f4ae78322c",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("a/file1.txt")).unwrap(),
+                object.state.get(&lpath("a/file1.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/a/file1.txt",
                 "7d9fe7396f8f5f9862bfbfff4d98877bf36cf4a44447078c8d887dcc2dab0497",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("a/file5.txt")).unwrap(),
+                object.state.get(&lpath("a/file5.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/a/d/e/file5.txt",
                 "4ccdbf78d368aed12d806efaf67fbce3300bca8e62a6f32716af2f447de1821e",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("a/b/file2.txt")).unwrap(),
+                object.state.get(&lpath("a/b/file2.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/a/b/file2.txt",
                 "b47592b10bc3e5c8ca8703d0862df10a6e409f43478804f93a08dd1844ae81b6",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("a/f/file6.txt")).unwrap(),
+                object.state.get(&lpath("a/f/file6.txt")).unwrap(),
                 &object.object_root,
                 "v4/content/a/f/file6.txt",
                 "df21fb2fb83c1c64015a00e7677ccceb8da5377cba716611570230fb91d32bc9",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("something/file1.txt")).unwrap(),
+                object.state.get(&lpath("something/file1.txt")).unwrap(),
                 &object.object_root,
                 "v1/content/a/file1.txt",
                 "7d9fe7396f8f5f9862bfbfff4d98877bf36cf4a44447078c8d887dcc2dab0497",
             );
             assert_file_details(
                 &s3_client,
-                object.state.get(&path("something/new.txt")).unwrap(),
+                object.state.get(&lpath("something/new.txt")).unwrap(),
                 &object.object_root,
                 "v3/content/something/new.txt",
                 "a253ff09c5a8678e1fd1962b2c329245e139e45f9cc6ced4e5d7ad42c4108fc0",
@@ -292,6 +293,223 @@ fn create_and_update_object() {
     );
 }
 
+#[test]
+fn validate_valid_object() {
+    skip_or_run_s3_test(
+        "validate_valid_object",
+        |_s3_client: S3Client, prefix: String, staging: TempDir, temp: TempDir| {
+            let repo = default_repo(&prefix, staging.path());
+            let object_id = "urn:example:rocfl:s3-object";
+            let commit_meta = CommitMeta::new()
+                .with_message(Some("commit".to_string()))
+                .with_user(
+                    Some("Peter Winckles".to_string()),
+                    Some("mailto:me@example.com".to_string()),
+                )
+                .unwrap();
+
+            repo.create_object(object_id, DigestAlgorithm::Sha512, "content", 0)
+                .unwrap();
+
+            create_dirs(&temp, "a/b/c");
+            create_dirs(&temp, "a/d/e");
+            create_dirs(&temp, "a/f");
+
+            create_file(&temp, "a/file1.txt", "File One");
+            create_file(&temp, "a/b/file2.txt", "File Two");
+            create_file(&temp, "a/b/file3.txt", "File Three");
+            create_file(&temp, "a/b/c/file4.txt", "File Four");
+            create_file(&temp, "a/d/e/file5.txt", "File Five");
+            create_file(&temp, "a/f/file6.txt", "File Six");
+
+            repo.move_files_external(object_id, &vec![temp.child("a").path()], "/")
+                .unwrap();
+
+            repo.commit(object_id, commit_meta.clone(), None, false)
+                .unwrap();
+
+            repo.remove_files(object_id, &vec!["a/b/file3.txt", "a/b/c/file4.txt"], false)
+                .unwrap();
+
+            repo.commit(object_id, commit_meta.clone(), None, false)
+                .unwrap();
+
+            repo.copy_files_internal(
+                object_id,
+                VersionNum::v1().into(),
+                &vec!["a/b/file3.txt"],
+                "/",
+                false,
+            )
+            .unwrap();
+            repo.copy_files_internal(
+                object_id,
+                VersionNum::v1().into(),
+                &vec!["a/file1.txt"],
+                "something/file1.txt",
+                false,
+            )
+            .unwrap();
+
+            create_dirs(&temp, "something");
+
+            repo.copy_files_external(
+                object_id,
+                &vec![create_file(&temp, "something/new.txt", "NEW").path()],
+                "something/new.txt",
+                true,
+            )
+            .unwrap();
+
+            repo.commit(object_id, commit_meta.clone(), None, false)
+                .unwrap();
+
+            repo.copy_files_external(
+                object_id,
+                &vec![create_file(&temp, "file6.txt", "UPDATED!").path()],
+                "a/f/file6.txt",
+                true,
+            )
+            .unwrap();
+
+            repo.move_files_internal(object_id, &vec!["a/d/e/file5.txt"], "a/file5.txt")
+                .unwrap();
+
+            repo.commit(object_id, commit_meta, None, false).unwrap();
+
+            let mut validator = repo.validate_repo(true).unwrap();
+
+            no_errors_storage(validator.storage_root_result());
+            no_warnings_storage(validator.storage_root_result());
+
+            for result in &mut validator {
+                let result = result.unwrap();
+                no_errors(&result);
+                no_warnings(&result);
+            }
+
+            no_errors_storage(validator.storage_hierarchy_result());
+            no_warnings_storage(validator.storage_hierarchy_result());
+        },
+    );
+}
+
+#[test]
+fn validate_invalid_object() {
+    skip_or_run_s3_test(
+        "validate_invalid_object",
+        |s3_client: S3Client, prefix: String, staging: TempDir, temp: TempDir| {
+            let repo = default_repo(&prefix, staging.path());
+            let object_id = "urn:example:rocfl:s3-object";
+            let object_id_2 = "urn:example:rocfl:s3-object-2";
+
+            let commit_meta = CommitMeta::new()
+                .with_message(Some("commit".to_string()))
+                .with_user(
+                    Some("Peter Winckles".to_string()),
+                    Some("mailto:me@example.com".to_string()),
+                )
+                .unwrap();
+
+            repo.create_object(object_id, DigestAlgorithm::Sha256, "content", 0)
+                .unwrap();
+
+            create_dirs(&temp, "a/b/c");
+            create_dirs(&temp, "a/d/e");
+            create_dirs(&temp, "a/f");
+
+            create_file(&temp, "a/file1.txt", "File One");
+            create_file(&temp, "a/b/file2.txt", "File Two");
+            create_file(&temp, "a/b/file3.txt", "File Three");
+            create_file(&temp, "a/b/c/file4.txt", "File Four");
+            create_file(&temp, "a/d/e/file5.txt", "File Five");
+            create_file(&temp, "a/f/file6.txt", "File Six");
+
+            repo.move_files_external(object_id, &vec![temp.child("a").path()], "/")
+                .unwrap();
+
+            repo.commit(object_id, commit_meta.clone(), None, false)
+                .unwrap();
+
+            repo.create_object(object_id_2, DigestAlgorithm::Sha512, "content", 0)
+                .unwrap();
+            repo.copy_files_external(
+                object_id_2,
+                &vec![create_file(&temp, "test.txt", "testing").path()],
+                "/",
+                false,
+            )
+            .unwrap();
+
+            repo.commit(object_id_2, commit_meta, None, false).unwrap();
+
+            let details = repo
+                .get_object_details(object_id, VersionRef::Head)
+                .unwrap();
+
+            write_file(&s3_client, &format!("{}/0=ocfl_1.0", prefix), "garbage");
+            write_file(&s3_client, &format!("{}/abc/random.txt", prefix), "garbage");
+            write_file(
+                &s3_client,
+                &format!("{}/file.txt", &details.object_root),
+                "garbage",
+            );
+            write_file(
+                &s3_client,
+                &format!("{}/v1/content/file.txt", &details.object_root),
+                "garbage",
+            );
+
+            let mut validator = repo.validate_repo(true).unwrap();
+
+            has_errors_storage(
+                validator.storage_root_result(),
+                &[ValidationError::new(
+                    ProblemLocation::StorageRoot,
+                    ErrorCode::E080,
+                    "Root version declaration is invalid. Expected: ocfl_1.0; Found: garbage"
+                        .to_string(),
+                )],
+            );
+            no_warnings_storage(validator.storage_root_result());
+
+            for result in &mut validator {
+                let result = result.unwrap();
+                match result.object_id.as_ref().unwrap().as_ref() {
+                    "urn:example:rocfl:s3-object" => {
+                        has_errors(&result, &[
+                            root_error(ErrorCode::E001, "Unexpected file in object root: file.txt"),
+                            root_error(ErrorCode::E023, "A content file exists that is not referenced in the manifest: v1/content/file.txt")
+                        ]);
+                        has_warnings(
+                            &result,
+                            &[root_warning(
+                                WarnCode::W004,
+                                "Inventory 'digestAlgorithm' should be 'sha512'. Found: sha256",
+                            )],
+                        );
+                    }
+                    "urn:example:rocfl:s3-object-2" => {
+                        no_errors(&result);
+                        no_warnings(&result);
+                    }
+                    id => panic!("Unexpected object: {}", id),
+                }
+            }
+
+            has_errors_storage(
+                validator.storage_hierarchy_result(),
+                &[ValidationError::new(
+                    ProblemLocation::StorageHierarchy,
+                    ErrorCode::E072,
+                    "Found a file in the storage hierarchy: abc/random.txt".to_string(),
+                )],
+            );
+            no_warnings_storage(validator.storage_hierarchy_result());
+        },
+    );
+}
+
 #[test]
 fn purge_object() {
     skip_or_run_s3_test(
@@ -312,11 +530,11 @@ fn purge_object() {
             repo.commit(object_id, CommitMeta::new(), None, false)
                 .unwrap();
 
-            let _ = repo.get_object(object_id, None).unwrap();
+            let _ = repo.get_object(object_id, VersionRef::Head).unwrap();
 
             repo.purge_object(object_id).unwrap();
 
-            match repo.get_object(object_id, None) {
+            match repo.get_object(object_id, VersionRef::Head) {
                 Err(RocflError::NotFound(_)) => (),
                 _ => panic!("Expected {} to not be found.", object_id),
             }
@@ -469,7 +687,7 @@ fn assert_file_details(
     content_path: &str,
     digest: &str,
 ) {
-    assert_eq!(path_rc(content_path), actual.content_path);
+    assert_eq!(cpath_rc(content_path), actual.content_path);
     assert_eq!(
         format!("{}/{}", object_root, content_path),
         actual.storage_path
@@ -542,6 +760,20 @@ fn get_content_with_key(s3_client: &S3Client, key: &str) -> String {
     })
 }
 
+fn write_file(s3_client: &S3Client, key: &str, contents: &str) {
+    tokio_test::block_on(async move {
+        let _ = s3_client
+            .put_object(PutObjectRequest {
+                bucket: bucket(),
+                key: key.to_string(),
+                body: Some(contents.to_string().into_bytes().into()),
+                ..Default::default()
+            })
+            .await
+            .expect(&format!("Expected put {} to succeed", key));
+    })
+}
+
 fn file_digest(s3_client: &S3Client, key: &str, algorithm: DigestAlgorithm) -> String {
     let content = get_content_with_key(s3_client, key);
     algorithm
